@@ -16,10 +16,12 @@
 
 import sys
 import re
+from os import path
 from calculate.install import distr
 from calculate.install.distr import DistributiveError
 from calculate.lib.datavars import (ReadonlyVariable, VariableError,
-                                    ReadonlyTableVariable, FieldValue)
+                                    ReadonlyTableVariable, FieldValue,
+                                    HumanReadable)
 from calculate.lib.utils.files import pathJoin
 from calculate.lib.utils.tools import ignore
 from calculate.lib.variables import env
@@ -38,20 +40,24 @@ class BuilderLinux(ReadonlyVariable):
     # variable for get current system info (example: os_linux_shortname)
     variable = ""
 
+    def get_fallback(self):
+        return ""
+
     def get(self):
         try:
             dv = self.Get('cl_builder_linux_datavars')
+            if dv:
+                return dv[self.variable]
         except VariableError:
-            return ""
-        if dv:
-            return dv[self.variable]
-        else:
-            return ""
+            pass
+        return self.get_fallback()
+
 
 class DataVarsBuilderLinux(linux.LinuxDataVars):
     """
     Упрощенная модель переменных для собираемого образа
     """
+
     def variables(self):
         l = super(DataVarsBuilderLinux, self).variables()
         return l + [linux.VariableClProfileSystem(),
@@ -87,8 +93,10 @@ class DataVarsBuilderLinux(linux.LinuxDataVars):
                     update.VariableClUpdateBinhostTime(section="update"),
                     update.VariableClUpdateBinhostTimeout(section="update"),
                     update.VariableClUpdateBinhostList(section="update"),
-                    update.VariableClUpdateBinhostTimestampPath(section="update"),
-                    update.VariableClUpdateBinhostRevisionPath(section="update"),
+                    update.VariableClUpdateBinhostTimestampPath(
+                        section="update"),
+                    update.VariableClUpdateBinhostRevisionPath(
+                        section="update"),
                     system.VariableOsArchMachineGentoo()]
 
     @classmethod
@@ -98,9 +106,9 @@ class DataVarsBuilderLinux(linux.LinuxDataVars):
         if m:
             map_arch = {'amd64': 'x86_64', 'x86': 'i686'}
             c = {'os_linux_build': m.group(3),
-                 'os_arch_machine':map_arch[m.group(2)],
+                 'os_arch_machine': map_arch[m.group(2)],
                  'os_linux_shortname': 'Gentoo',
-                 'os_linux_subname': "Stage%s"%m.group(1),
+                 'os_linux_subname': "Stage%s" % m.group(1),
                  'cl_profile_name': 'Gentoo profile',
                  'os_linux_name': 'Gentoo'}
         else:
@@ -119,10 +127,12 @@ class DataVarsBuilderLinux(linux.LinuxDataVars):
     def __repr__(self):
         return "Builder variables"
 
+
 class VariableClBuilderBinarySet(ReadonlyVariable):
     """
     Собираемый дистрибутив - бинарный FEATURES="getbinpkg"
     """
+
     def get(self):
         dv = self.Get('cl_builder_linux_datavars')
         if dv:
@@ -130,10 +140,12 @@ class VariableClBuilderBinarySet(ReadonlyVariable):
                 return "on"
         return "off"
 
+
 class VariableClBuilderLinuxDatavars(ReadonlyVariable):
     """
     Переменные дистрибутива
     """
+
     def source_data(self):
         image = self.Get('cl_builder_source')
         image_fn = self.Get('cl_builder_source_filename')
@@ -182,6 +194,7 @@ class VariableOsBuilderLinuxVer(BuilderLinux):
     def init(self):
         self.label = _("Building system version")
 
+
 class VariableOsBuilderLinuxBuild(BuilderLinux):
     """Shortname of system"""
     variable = "os_linux_build"
@@ -202,6 +215,15 @@ class VariableOsBuilderArchMachine(BuilderLinux):
     """Shortname of system"""
     variable = "os_arch_machine"
 
+    def get_fallback(self):
+        prefix = self.Get('cl_builder_path')
+        if prefix:
+            if path.exists(path.join(prefix, 'lib64')):
+                return 'x86_64'
+            else:
+                return 'i686'
+        return ""
+
 
 class VariableOsBuilderLinuxFiles(BuilderLinux):
     """Shortname of system"""
@@ -226,6 +248,7 @@ class VariableOsBuilderLinuxSystem(BuilderLinux):
     """
     Install system name
     """
+
     def init(self):
         self.label = _("System type")
 
@@ -252,6 +275,7 @@ class VariableOsBuilderLinuxFullname(ReadonlyVariable):
         return self.fullname_format.format(
             name=name, ver=ver, subname=subname, arch=arch, build=build)
 
+
 class VariableOsBuilderLinuxSubname(BuilderLinux):
     """
     Install subname
@@ -264,16 +288,19 @@ class VariableOsBuilderLinuxSubname(BuilderLinux):
     def humanReadable(self):
         return self.Get() or _("no")
 
+
 class VariableClBuilderPortdirOverlay(ReadonlyVariable):
     """
     Пути оверлеев в собираемом образе относительно точки монтирования
     """
+
     def get(self):
         return [location
                 for name, location in
                 filter(None, self.Get('builder.cl_builder_repository_data'))
                 if name not in ("portage", "gentoo")]
 
+
 class VariableClBuilderOtherRepData(update.VariableClUpdateOtherRepData):
     """
     Информация о прочих репозиториях
@@ -283,7 +310,8 @@ class VariableClBuilderOtherRepData(update.VariableClUpdateOtherRepData):
 
     portdir_overlay = "builder.cl_builder_portdir_overlay"
 
-class VariableClBuilderOtherRepName(FieldValue,ReadonlyVariable):
+
+class VariableClBuilderOtherRepName(FieldValue, ReadonlyVariable):
     """
     Список имен прочих репозиториев
     """
@@ -291,7 +319,8 @@ class VariableClBuilderOtherRepName(FieldValue,ReadonlyVariable):
     source_variable = "builder.cl_builder_other_rep_data"
     column = 0
 
-class VariableClBuilderOtherRepPath(FieldValue,ReadonlyVariable):
+
+class VariableClBuilderOtherRepPath(FieldValue, ReadonlyVariable):
     """
     Список путей до прочих репозиториев
     """
@@ -305,7 +334,7 @@ class VariableClBuilderRepositoryData(ReadonlyTableVariable):
     """
     source = ["cl_repository_name", "cl_repository_location"]
 
-    def get(self, hr=False):
+    def get(self, hr=HumanReadable.No):
         try:
             dv = self.Get('cl_builder_linux_datavars')
             if dv:
@@ -314,20 +343,23 @@ class VariableClBuilderRepositoryData(ReadonlyTableVariable):
             pass
         return [[]]
 
-class VariableClBuilderRepositoryName(FieldValue,ReadonlyVariable):
+
+class VariableClBuilderRepositoryName(FieldValue, ReadonlyVariable):
     """
     """
     type = "list"
     source_variable = "cl_builder_repository_data"
     column = 0
 
-class VariableClBuilderRepositoryLocation(FieldValue,ReadonlyVariable):
+
+class VariableClBuilderRepositoryLocation(FieldValue, ReadonlyVariable):
     """
     """
     type = "list"
     source_variable = "cl_builder_repository_data"
     column = 1
 
+
 class VariableClBuilderSyncOverlayRep(ReadonlyVariable):
     """
     Обновляемые репозитории (исключая portage)
@@ -346,10 +378,12 @@ class VariableClBuilderSyncOverlayRep(ReadonlyVariable):
         else:
             return []
 
+
 class VariableClBuilderLinuxDistdir(ReadonlyVariable):
     """
     Полный путь до DISTDIR собираемой системы относительно текущего корня
     """
+
     def get(self):
         fallback_distdir = '/var/calculate/remote/distfiles'
         dv = self.Get('cl_builder_linux_datavars')
@@ -358,10 +392,12 @@ class VariableClBuilderLinuxDistdir(ReadonlyVariable):
             return pathJoin(builder_path, dv.Get('cl_distdir'))
         return pathJoin(builder_path, fallback_distdir)
 
+
 class VariableClBuilderLinuxPkgdir(ReadonlyVariable):
     """
     Полный путь до PKGDIR собираемой системы относительно текущего корня
     """
+
     def get(self):
         fallback_pkgdir = '/var/calculate/remote/packages'
         dv = self.Get('cl_builder_linux_datavars')
